#include "Joystick.h"
#include <windows.h>
#include <mmsystem.h>
#include <assert.h>

Joystick::Joystick ( int id, int maxDir [ 6 ], int minDir [ 6 ], int buttons )
{
	this->id = id;
	memcpy ( this->maxDir, maxDir, sizeof ( int ) * 6 );
	memcpy ( this->minDir, minDir, sizeof ( int ) * 6 );
	this->buttons = buttons;
}

Joystick::Joystick ( void )
{
	// invalid joystick
	id = -1;
	buttons = -1;
	memset ( minDir, 0, sizeof ( int ) * 6 );
	memset ( maxDir, 0, sizeof ( int ) * 6 );
}

int Joystick::numButtons ( void ) const
{
	return buttons;
}

bool Joystick::getState ( int &buttons, float pos [ 6 ], float &pov ) const
{
	JOYINFOEX joyinfo;

	assert ( id != -1 );

	joyinfo.dwSize = sizeof ( JOYINFOEX );
	joyinfo.dwFlags = JOY_RETURNALL;
	if ( joyGetPosEx ( id, &joyinfo ) == JOYERR_NOERROR )
	{
		if ( joyinfo.dwPOV == 65535 )
		{
			pov = -1;
		}
		else
		{
			pov = joyinfo.dwPOV / 100.0f; // accurate to 1/100 of a degree.. kind of
		}

		// invert y so that up is positive
		pos [ 0 ] = 2 * ( joyinfo.dwXpos - ( minDir [ 0 ] + maxDir [ 0 ] ) / 2.0f ) / (float)( maxDir [ 0 ] - minDir [ 0 ] );
		pos [ 1 ] = -2 * ( joyinfo.dwYpos - ( minDir [ 1 ] + maxDir [ 1 ] ) / 2.0f ) / (float)( maxDir [ 1 ] - minDir [ 1 ] );
		pos [ 2 ] = 2 * ( joyinfo.dwZpos - ( minDir [ 2 ] + maxDir [ 2 ] ) / 2.0f ) / (float)( maxDir [ 2 ] - minDir [ 2 ] );
		pos [ 3 ] = -2 * ( joyinfo.dwRpos - ( minDir [ 3 ] + maxDir [ 3 ] ) / 2.0f ) / (float)( maxDir [ 3 ] - minDir [ 3 ] );
		pos [ 4 ] = 2 * ( joyinfo.dwUpos - ( minDir [ 4 ] + maxDir [ 4 ] ) / 2.0f ) / (float)( maxDir [ 4 ] - minDir [ 4 ] );
		pos [ 5 ] = -2 * ( joyinfo.dwVpos - ( minDir [ 5 ] + maxDir [ 5 ] ) / 2.0f ) / (float)( maxDir [ 5 ] - minDir [ 5 ] );

		buttons = joyinfo.dwButtons;
		return true;
	}
	return false;
}

// returns number of joysticks connected and fills sticks with the joystick info
int JoystickInfo::getJoySticks ( std::vector<Joystick> &sticks )
{
	JOYCAPS joyinfo;

	sticks.clear ( );
	for ( int i = 0; i < joyGetNumDevs ( ); i++ )
	{
		if ( joyGetDevCaps ( i, &joyinfo, sizeof ( JOYCAPS ) ) == JOYERR_NOERROR )
		{
			int maxDir [ 6 ], minDir [ 6 ];
			minDir [ 0 ] = joyinfo.wXmin;
			minDir [ 1 ] = joyinfo.wYmin;
			minDir [ 2 ] = joyinfo.wZmin;
			minDir [ 3 ] = joyinfo.wRmin;
			minDir [ 4 ] = joyinfo.wUmin;
			minDir [ 5 ] = joyinfo.wVmin;
			maxDir [ 0 ] = joyinfo.wXmax;
			maxDir [ 1 ] = joyinfo.wYmax;
			maxDir [ 2 ] = joyinfo.wZmax;
			maxDir [ 3 ] = joyinfo.wRmax;
			maxDir [ 4 ] = joyinfo.wUmax;
			maxDir [ 5 ] = joyinfo.wVmax;

			sticks.push_back ( Joystick ( i, maxDir, minDir, joyinfo.wNumButtons ) );
		}
	}

	return sticks.size ( );
}
