#include <GL/glut.h>
#include <math.h>
#include "Joystick.h"
#include <vector>

using namespace std;

vector<Joystick> sticks;

void drawCrossHairs ( float size = 0.03f )
{
	glBegin ( GL_LINES );
		glVertex2f ( size, 0 );
		glVertex2f ( -size, 0 );
		glVertex2f ( 0, size );
		glVertex2f ( 0, -size );
	glEnd ( );
}

// Draws the current IAT
void display ( void )
{
	float pos [ 6 ];
	float pov;
	int buttons;

	// not necessary to call this function repeatedly unless you 
	// think user will plug and unplug joystick during gameplay
	JoystickInfo::getJoySticks ( sticks );

	glClear ( GL_COLOR_BUFFER_BIT );
	for ( int i = 0; i < sticks.size ( ); i++ )
	{
		if ( sticks [ i ].getState ( buttons, pos, pov ) )
		{
			glBegin ( GL_POINTS );
			for ( int j = 0; j < sticks [ i ].numButtons ( ); j++ )
			{
				if ( ( 0x01 << j ) & buttons )
				{
					glColor3f ( 1, 1, 0 );
				}
				else
				{
					glColor3f ( .5, .5, .5 );
				}
				float theta = ( 2 * 3.14159265 * j ) / sticks [ i ].numButtons ( ) ;
				float r = 0.5 + ( 0.5 * i ) / sticks.size ( );  // change the radius for each stick
				glVertex2f ( r * cos ( theta ), r * sin ( theta ) );
			}
			glEnd ( );
			glPushMatrix ( );
				glTranslatef ( pos [ 0 ], pos [ 1 ], 0 );
				glColor3f ( 1, .3, .3 );
				drawCrossHairs ( );
			glPopMatrix ( );
			glPushMatrix ( );
				glTranslatef ( pos [ 2 ], pos [ 3 ], 0 );
				glColor3f ( .3, 1, .3 );
				drawCrossHairs ( );
			glPopMatrix ( );
			glPushMatrix ( );
				if ( pov != -1 )
				{
					glTranslatef ( .7 * cos ( pov * 3.1415926 / 180 ), .7 * sin ( pov * 3.1415926 / 180 ), 0 );
				}
				glColor3f ( .3, .3, 1 );
				drawCrossHairs ( );
			glPopMatrix ( );
		}
	}
	glFlush ( );
	glutSwapBuffers ( );
}


/* do not modify the reshape function */
void reshape ( int width, int height )
{
	glViewport ( 0, 0, width, height );
	glMatrixMode ( GL_PROJECTION );
	glLoadIdentity ( );    
	gluOrtho2D (-1, 1, -1, 1);
	glMatrixMode ( GL_MODELVIEW );
    glLoadIdentity ( );
}

void init ( void )
{
	JoystickInfo::getJoySticks ( sticks );
	glPointSize ( 10 );
}

void idle ( void )	
{
	glutPostRedisplay ( );
/*	float pos [ 6 ];
	float pov;
	int buttons;

	system ( "cls" );
	for ( int i = 0; i < sticks.size ( ); i++ )
	{
		if ( sticks [ i ].getState ( buttons, pos, pov ) )
		{
			printf ( "{x,y,z,r,u,v} = {%.2f,%.2f,%.2f,%.2f,%.2f,%.2f}\n", pos [ 0 ], pos [ 1 ], pos [ 2 ], pos [ 3 ], pos [ 4 ], pos [ 5 ] );
			printf ( "buttons = %x\n", buttons );
			printf ( "pov = %f\n", pov );
		}
	}*/
}

int main ( int argc, char** argv )
{
	glutInit ( &argc, argv );
	glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB );
	glutInitWindowSize ( 500, 500 );
	glutInitWindowPosition ( 100, 100 );
	glutCreateWindow ( "Joystick Test" );
	glutDisplayFunc ( display );
	glutReshapeFunc ( reshape );
	glutIdleFunc ( idle );

	init ( );

	glutMainLoop ( );
	return 0;
}
